<?php

namespace Ipol\Catapulto\Wordpress;

use Automattic\WooCommerce\StoreApi\Schemas\V1\CheckoutSchema;
use Automattic\WooCommerce\Blocks\Integrations\IntegrationRegistry;
use Automattic\WooCommerce\Utilities\FeaturesUtil;
use Ipol\Catapulto\Wordpress\Controller\MainController;
use Ipol\Catapulto\Wordpress\CronTasks\UpdateOrderStatuses;
use Ipol\Catapulto\Wordpress\DB\CartVariantsDB;
use Ipol\Catapulto\Wordpress\DB\DBTools;
use Ipol\Catapulto\Wordpress\DB\OrdersDB;
use Ipol\Catapulto\Wordpress\Frontend\CheckoutBlock;
use Ipol\Catapulto\Wordpress\Rest\RestHandler;
use Ipol\Catapulto\Wordpress\Shipping\Calculator;
use Ipol\Catapulto\Wordpress\Tools\TemplateRender;
use Ipol\Catapulto\Wordpress\Tools\Variables;
use Ipol\Catapulto\Wordpress\Tools\VariantStore;

class CatapultoPlugin
{
    public const PLUGIN_ID = 'catapultodelivery';
    public const PLUGIN_MENU_ID = 'ipol_catapulto_menu';
    private static $mainPluginFile = '';
    private static $mainPluginDir = '';
    private static $mainPluginUrl = '';
    private $pluginBaseName;

    //OtherConsts
    public const POA_DOCUMENT_TITLE = 'Доверенность на отправку';

    public static function getMainPluginDir():string
    {
        return self::$mainPluginDir;
    }

    public function __construct(string $mainPluginFile)
    {
        self::$mainPluginFile = $mainPluginFile;
        self::$mainPluginDir = dirname($mainPluginFile);
        self::$mainPluginUrl = plugin_dir_url($mainPluginFile);
        $this->pluginBaseName = plugin_basename($mainPluginFile);

        (new Variables())
            ->setMainPluginDir(self::$mainPluginDir)
            ->setMainPluginFile(self::$mainPluginFile)
            ->setMainPluginUrl(self::$mainPluginUrl)
            ->setPliginVersion(get_file_data($mainPluginFile, ['Version'])[0]);

        $this->wpInit();
    }

    private function wpInit()
    {
        add_action('activate_' . $this->pluginBaseName, [self::class, 'activate']);
        add_action('deactivate_' . $this->pluginBaseName, [self::class, 'deactivate']);

        /*add_action('plugins_loaded', static function(){
            load_plugin_textdomain(self::PLUGIN_ID, false, $this->mainPluginDir . '/lang');
        });*/

        add_action('before_woocommerce_init', static function () {
            if (class_exists(FeaturesUtil::class)) {
                FeaturesUtil::declare_compatibility('custom_order_tables', self::$mainPluginFile, true);
                FeaturesUtil::declare_compatibility('cart_checkout_blocks', self::$mainPluginFile, true);
            }
        });

        add_action('woocommerce_shipping_methods', static function($methods) {
            $methods[CatapultoPlugin::PLUGIN_ID] = CatapultoShippingMethod::class;
            return $methods;
        });

        add_action('wp_head', static function () {
            if (function_exists('wp_enqueue_script_module') && class_exists(FeaturesUtil::class)) {
                wp_enqueue_script_module('catapulto_widget', 'https://widgetcdn.catapulto.ru/assets/js/catapulto-widget/v3/catapulto-widget.js',[]);
            }
        });

        add_action('admin_enqueue_scripts', static function(){
            wp_enqueue_script(
                'admin',
                self::$mainPluginUrl . 'assets/js/admin.js',
                ['jquery']
            );
            wp_enqueue_style('admin', self::$mainPluginUrl . 'assets/css/admin.css');
            ?>
            <script type="module" src="https://widgetcdn.catapulto.ru/assets/js/catapulto-widget/v3/catapulto-widget.js"></script>
            <?php
        });

        //Frontend for new woocommerce
        add_action('woocommerce_blocks_loaded', static function() {
            if (class_exists(FeaturesUtil::class)) {
                add_action('woocommerce_blocks_checkout_block_registration', static function (IntegrationRegistry $registry) {
                    $registry->register(new CheckoutBlock);
                });
                woocommerce_store_api_register_update_callback(
                    [
                        'namespace' => 'catapultodelivery',
                        'callback'  => function ($data) {
                            $cartVariants = new CartVariantsDB();
                            $orderId = 0;
                            if ( isset($data['ctpt_params']) && isset($data['ctpt_params']['order_id'])) $orderId = intval($data['ctpt_params']['order_id']);
                            $savedData = [
                                'dadata' => $data['dadata'] ?? [],
                                'variant' => $data['variant'] ?? [],
                                'selected' => $data['selected'] ?? [],
                                'rate_param' => $data['rate_param'] ?? '',
                                'Terminal' => $data['Terminal'] ?? [],
                                'ctpt_params' => $data['ctpt_params'] ?? [],
                            ];
                            if ($orderId > 0) $cartVariants->saveCartVariant($orderId, $savedData);
                            VariantStore::setVariantStatic($data);
                            WC()->cart->calculate_shipping();
                        }
                    ]
                );
                woocommerce_store_api_register_endpoint_data(
                    array(
                        'endpoint'        => CheckoutSchema::IDENTIFIER,
                        'namespace'       => self::PLUGIN_ID,
                        'schema_callback' => static function () {
                            return [
                                'ctpt_ratedata' => [
                                    'description' => 'Данные о выбранном тарифе Catapulto',
                                    'type'        => ['string', 'null'],
                                    'readonly'    => true,
                                ],
                                'ctpt_paychanged' => [
                                    'description' => 'Изменение способа оплаты после выбора тарифа доставки',
                                    'type'        => ['string', 'null'],
                                    'readonly'    => true,
                                ],
                            ];
                        },
                        'schema_type'     => ARRAY_A,
                    )
                );
            }
        });

        add_action('rest_api_init', static function () {
            register_rest_route(self::PLUGIN_ID, '/widgethandler', [
                'methods'             => \WP_REST_Server::EDITABLE,
                'callback'            => [RestHandler::class, 'widgetHandler'],
                'permission_callback' => static fn() => true,
                'show_in_index'       => true,
                'args'                => [],
            ]);
            register_rest_route(self::PLUGIN_ID, '/opsync', [
                'methods'             => \WP_REST_Server::EDITABLE,
                'callback'            => [RestHandler::class, 'syncOperators'],
                'permission_callback' => function() {
                    $currentUser = wp_get_current_user();
                    return in_array('administrator',$currentUser->roles);
                },
                'show_in_index'       => true,
                'args'                => [],
            ]);

            //Catapulto panel in Order Admin Page
            register_rest_route(self::PLUGIN_ID, '/reselectrate', [
                'methods'             => \WP_REST_Server::EDITABLE,
                'callback'            => [RestHandler::class, 'orderReselectWidgetRate'],
                'permission_callback' => function() {
                    $currentUser = wp_get_current_user();
                    return in_array('administrator',$currentUser->roles);
                },
                'show_in_index'       => true,
                'args'                => [],
            ]);

            register_rest_route(self::PLUGIN_ID, '/ordersend', [
                'methods'             => \WP_REST_Server::EDITABLE,
                'callback'            => [RestHandler::class, 'sendOrder'],
                'permission_callback' => function() {
                    $currentUser = wp_get_current_user();
                    return in_array('administrator',$currentUser->roles);
                },
                'show_in_index'       => true,
                'args'                => [],
            ]);

            register_rest_route(self::PLUGIN_ID, '/orderrenew', [
                'methods'             => \WP_REST_Server::EDITABLE,
                'callback'            => [RestHandler::class, 'renewOrder'],
                'permission_callback' => function() {
                    $currentUser = wp_get_current_user();
                    return in_array('administrator',$currentUser->roles);
                },
                'show_in_index'       => true,
                'args'                => [],
            ]);

            register_rest_route(self::PLUGIN_ID, '/ordercancel', [
                'methods'             => \WP_REST_Server::EDITABLE,
                'callback'            => [RestHandler::class, 'cancelOrder'],
                'permission_callback' => function() {
                    $currentUser = wp_get_current_user();
                    return in_array('administrator',$currentUser->roles);
                },
                'show_in_index'       => true,
                'args'                => [],
            ]);

            //Mass actions on Catapulto Orders Page
            register_rest_route(self::PLUGIN_ID, '/pg_checkstatus', [
                'methods'             => \WP_REST_Server::EDITABLE,
                'callback'            => [RestHandler::class, 'massRenewStatuses'],
                'permission_callback' => function() {
                    $currentUser = wp_get_current_user();
                    return in_array('administrator',$currentUser->roles);
                },
                'show_in_index'       => true,
                'args'                => [],
            ]);

            register_rest_route(self::PLUGIN_ID, '/pg_cancelorders', [
                'methods'             => \WP_REST_Server::EDITABLE,
                'callback'            => [RestHandler::class, 'massCancelOrders'],
                'permission_callback' => function() {
                    $currentUser = wp_get_current_user();
                    return in_array('administrator',$currentUser->roles);
                },
                'show_in_index'       => true,
                'args'                => [],
            ]);

            register_rest_route(self::PLUGIN_ID, '/pg_printdocs', [
                'methods'             => \WP_REST_Server::EDITABLE,
                'callback'            => [RestHandler::class, 'massGetPrintDocs'],
                'permission_callback' => function() {
                    $currentUser = wp_get_current_user();
                    return in_array('administrator',$currentUser->roles);
                },
                'show_in_index'       => true,
                'args'                => [],
            ]);

            // For old version WP
            register_rest_route(self::PLUGIN_ID, '/wpold_ctptsave', [
                'methods'             => \WP_REST_Server::EDITABLE,
                'callback'            => [RestHandler::class, 'saveCtptrateFromCheckout'],
                'permission_callback' => static fn() => true,
                'show_in_index'       => true,
                'args'                => [],
            ]);
        });

        add_action('woocommerce_store_api_checkout_update_customer_from_request', [self::class, 'saveCustomerData'], 10, 2);

        add_action('woocommerce_store_api_checkout_update_order_from_request', [self::class, 'saveOrderData'], 10, 2);

        add_action('admin_menu', static function(){
            add_submenu_page(
                'woocommerce',
                'Управление заявками Catapulto',
                'Доставка Catapulto',
                'manage_options',
                self::PLUGIN_MENU_ID,
                static function() {
                    $limit = 20;
                    $pageNum = 1;
                    if (isset($_GET['paged'])) $pageNum = intval(sanitize_text_field($_GET['paged']));

                    $orderBy = 'id';
                    if (isset($_GET['orderby'])) $orderBy = sanitize_text_field($_GET['orderby']);
                    $orderDesc = 'desc';
                    if (isset($_GET['order'])) $orderDesc = sanitize_text_field($_GET['order']);
                    $queryParams = [];
                    if ($orderBy != 'id') $queryParams['orderby'] = $orderBy;
                    if ($orderDesc != 'desc') $queryParams['order'] = $orderDesc;

                    $ordersDB = new OrdersDB();
                    $counts = $ordersDB->getPagesRecordsCount($limit);

                    if ($pageNum > $counts[1]) $pageNum = $counts[1];
                    if ($pageNum < 1) $pageNum = 1;
                    $offset = ($pageNum-1) * $limit;

                    $columnData = [
                        'updated' => [
                            'name' => 'Дата изменения',
                            'sortable' => true,
                        ],
                        'order_id' => [
                            'name' => 'Заказ',
                            'sortable' => true,
                        ],
                        'number' => [
                            'name' => 'Номер заказа Catapulto',
                            'sortable' => true,
                        ],
                        'tracking_number' => [
                            'name' => 'Трек-номер',
                        ],
                        'tracking_link' => [
                            'name' => 'Ссылка на трекинг',
                        ],
                        'operator' => [
                            'name' => 'Служба доставки',
                        ],
                        'main_status_display' => [
                            'name' => 'Статус',
                        ],
                        'pickup_day' => [
                            'name' => 'Дата забора',
                        ],
                        'delivery_day' => [
                            'name' => 'Дата доставки',
                        ],
                        'price' => [
                            'name' => 'Стоимость',
                        ],
                        'weight' => [
                            'name' => 'Вес',
                        ],
                        'receiver_locality' => [
                            'name' => 'Город получатель',
                        ],
                        'receiver_name' => [
                            'name' => 'Имя получателя',
                        ],
                        'receiver_company' => [
                            'name' => 'Компания-получатель',
                        ],
                        'receiver_phone' => [
                            'name' => 'Телефон получателя',
                        ],
                        'description' => [
                            'name' => 'Описание',
                        ],
                        'with_insurance' => [
                            'name' => 'Страховка',
                        ],
                        'is_pod' => [
                            'name' => 'Наложенный платеж',
                        ],
                        'sum_to_pay' => [
                            'name' => 'Сумма к оплате',
                        ],
                        'problem_text' => [
                            'name' => 'Сист сообщение',
                        ],
                    ];

                    $tableData = $ordersDB->getAll($orderBy, $orderDesc, $offset, $limit);

                    $templateRender = new TemplateRender();
                    $paginatorHtml = $templateRender->renderPaginatorBlock($pageNum, $counts[1], $counts[0], $queryParams);
                    $tableContent = $templateRender->renderTable($columnData, $tableData, $orderBy, $orderDesc);

                    $templateRender->render('adminpage/page.tpl', [
                        'nonce' => wp_create_nonce('wp_rest'),
                        'paginator' => $paginatorHtml,
                        'table' => $tableContent,
                    ], true);
                }
            );
        });

        add_action('add_meta_boxes', function() {
            $screen = class_exists(\Automattic\WooCommerce\Internal\DataStores\Orders\CustomOrdersTableController::class) && wc_get_container()->get(\Automattic\WooCommerce\Internal\DataStores\Orders\CustomOrdersTableController::class)->custom_orders_table_usage_is_enabled() ? wc_get_page_screen_id('shop_order') : 'shop_order';

            add_meta_box(self::PLUGIN_ID,'Доставка Catapulto',['\Ipol\Catapulto\Wordpress\Frontend\AdminPages', 'renderOrderMetaBox'], $screen, 'side', 'high');
        });

        add_action('ctpt_checkstatuses', [UpdateOrderStatuses::class, 'updateStatuses']);

        //For old version WP/WC
        add_filter('woocommerce_cart_shipping_method_full_label', [$this, 'actionCartShippingMethodFullLabel'], 10, 2);
        add_action('woocommerce_after_shipping_rate', [$this, 'actionAfterShippingRate'], 10, 2);
        add_action('woocommerce_after_checkout_validation', [$this, 'actionWoocommerceAfterCheckoutValidation'], 10, 2);
        add_action('woocommerce_checkout_create_order', [$this, 'actionWoocommerceCheckoutCreateOrder'], 10, 2);
        add_action('woocommerce_checkout_update_order_review', static function(){
            $packages = WC()->cart->get_shipping_packages();
            foreach ($packages as $key => $value) {
                $shipping_session = "shipping_for_package_$key";
                unset(WC()->session->$shipping_session);
            }
        }, 10);

    }

    public static function activate()
    {
        $dbTools = new DBTools();
        $dbTools->createTablesStructure();

        wp_clear_scheduled_hook('ctpt_checkstatuses');
        wp_schedule_event(time(), 'hourly', 'ctpt_checkstatuses');
    }

    public static function deactivate()
    {
        wp_clear_scheduled_hook('ctpt_checkstatuses');
    }

    public static function saveCustomerData(\WC_Customer $customer, \WP_REST_Request $request): void
    {
        $chosenMethod = WC()->session->get('chosen_shipping_methods')[0];
        $isCatapultoDLV = preg_match("/^".self::PLUGIN_ID."/", $chosenMethod);
        if ($isCatapultoDLV == 0) return;

        //get noPVZnoOrder module option
        $shippingMethod = new CatapultoShippingMethod();
        $isEnableNoRate = $shippingMethod->isEnabledEmptyRateOrder();
        $needReselectAfterChangePS = $shippingMethod->isNeedReselectAfterChangePS();

        //get Extension Data
        $jsonData = $request->get_json_params();
        $rateData = null;
        $payChanged = null;
        if (
            isset($jsonData['extensions'])
            && isset($jsonData['extensions'][self::PLUGIN_ID])
        ) {
            $rData = [];
            if (!empty($jsonData['extensions'][CatapultoPlugin::PLUGIN_ID]['ctpt_ratedata'])) {
                $rateData = $jsonData['extensions'][CatapultoPlugin::PLUGIN_ID]['ctpt_ratedata'];
                if (!empty($rateData)) $rData = json_decode($rateData, true);
            }
            if (!empty($jsonData['extensions'][CatapultoPlugin::PLUGIN_ID]['ctpt_paychanged'])) $payChanged = $jsonData['extensions'][CatapultoPlugin::PLUGIN_ID]['ctpt_paychanged'] === 'true';
            if (
                !$payChanged
                && is_array($rData)
                && isset($rData['ctpt_params'])
                && isset($rData['ctpt_params']['ps'])
            ) {
                $payChanged = $jsonData['payment_method'] != $rData['ctpt_params']['ps'];
            }
        }

        if (!$isEnableNoRate) {
            if ( ($rateData === null) || ($payChanged === null) ) throw new \WC_REST_Exception('500','Необходимо выбрать вариант доставки Catapulto', 500);
            if ($needReselectAfterChangePS && $payChanged) throw new \WC_REST_Exception('500','После изменения способа оплаты Необходимо выбрать новый вариант доставки Catapulto', 500);
        }
    }

    public static function saveOrderData(\WC_Order $order, \WP_REST_Request $request): void
    {
        $shippingMethodArray = $order->get_shipping_methods();
        $method = array_shift($shippingMethodArray);
        if ($method->get_method_id() != self::PLUGIN_ID) return;

        $emptyRateData = [
            'dadata' => [
                'data' => [],
            ],
            'variant' => [
                'additional_services' => [],
                'operator' => 'NotSet',
            ],
            'ctpt_params' => [
                'isEmpty' => true,
            ],
        ];

        $jsonData = $request->get_json_params();
        $rateDataString = null;
        if (
            isset($jsonData['extensions'][CatapultoPlugin::PLUGIN_ID])
            && isset($jsonData['extensions'][CatapultoPlugin::PLUGIN_ID]['ctpt_ratedata'])
        ) $rateDataString = $jsonData['extensions'][CatapultoPlugin::PLUGIN_ID]['ctpt_ratedata'];
        $rateData = json_decode($rateDataString, true);
        if (!$rateData) $rateDataString = json_encode($emptyRateData);
        $order->add_meta_data('ctpt_ratedata', $rateDataString);
    }

    ///FOR OLD WP VERSIONS
    public function actionAfterShippingRate($method, $index) {
        if(!is_checkout()) return;
        if ($method->id != self::PLUGIN_ID . ':pickup') return;
        $chosen_method_id = WC()->session->chosen_shipping_methods[$index];

        if($chosen_method_id == self::PLUGIN_ID.':pickup'):
            parse_str(!empty($_POST['post_data']) ? sanitize_text_field($_POST['post_data']) : '', $post_data);
            woocommerce_form_field( 'ctpt_selected_rate' , [
                'type'          => 'hidden',
                'required'      => true
            ], !empty($post_data['ctpt_selected_rate']) ? $post_data['ctpt_selected_rate'] : '');
            woocommerce_form_field( 'ctpt_changed_ps' , [
                'type'          => 'hidden',
                'required'      => true
            ], !empty($post_data['ctpt_changed_ps']) ? $post_data['ctpt_changed_ps'] : '');
        endif;
    }

    public function actionCartShippingMethodFullLabel(string $label, \WC_Shipping_Rate $method): string {
        if(function_exists('is_checkout') && is_checkout()) {
            add_action('wp_footer', static function () {
                if (!is_checkout()) return;
                ?>
                <script type="module" src="https://widgetcdn.catapulto.ru/assets/js/catapulto-widget/v3/catapulto-widget.js"></script>
                <?php
                wp_enqueue_script('ctpt_widget_handler', self::$mainPluginUrl . 'assets/js/widget_handler.js', ['jquery']);
            });
            wp_enqueue_style('checkout', self::$mainPluginUrl . 'assets/css/old_checkout.css');

            if ($method->id === self::PLUGIN_ID . ':pickup') {
                $selectedShippingMethod = WC()->session->get( 'chosen_shipping_methods' );
                $selectedPaymentMethod = WC()->session->get( 'chosen_payment_method' );

                $sess = WC()->session;
                $cart = WC()->cart->get_cart();
                $items = WC()->checkout->get_checkout_fields();

                if (!empty($selectedShippingMethod)) $selectedShippingMethod = array_shift($selectedShippingMethod);
                if ($selectedShippingMethod === self::PLUGIN_ID . ':pickup') {
                    $shippingMethod = new CatapultoShippingMethod();
                    $settings = $shippingMethod->getAllSettings();
                    $cargo = Calculator::getCurrentCargoDataByCart();

                    $mainController = new MainController(
                        $shippingMethod->getApiKey(),
                        $shippingMethod->getCustomApiURL(),
                        15,
                        null,
                        null,
                        null
                    );

                    $widgetDeliveryFrom = $settings['widgetDeliveryFrom'];
                    $operatorsSettings = [];
                    $inverseOperators = [];
                    $freeDeliveryOperators = [];
                    $icons = [];
                    $companyIcons = $mainController->getCompanyIcons();
                    if ($companyIcons) {
                        $elements = $companyIcons->getCompanies();
                        while ($el = $elements->getNext()) {
                            $icons[$el->getOperatorId()] = $el->getIcon();

                            $operatorSetting = [
                                'operator' => $el->getOperatorId(),
                                'inverse_delivery_type' => false,
                                'free_delivery' => false,
                            ];

                            if ($settings['op_type_' . $el->getOperatorId()] != $widgetDeliveryFrom ) {
                                $inverseOperators[] = $el->getOperatorId();
                                $operatorSetting['inverse_delivery_type'] = true;
                            }

                            if ($settings['op_free_' . $el->getOperatorId()] === 'yes' ) {
                                $freeDeliveryOperators[] = $el->getOperatorId();
                                $operatorSetting['free_delivery'] = true;
                            }
                            $operatorsSettings[] = $operatorSetting;
                        }
                    }

                    $widgetDeliveryTypes = "only_delivery_type: '{$settings['widgetDeliveryTypes']}',";
                    if ($settings['widgetDeliveryTypes'] == 'All') $widgetDeliveryTypes = '';

                    $customWSS = '';
                    if ($settings['isTest'] === 'yes') $customWSS = "ws_api_domain: '{$settings['customWSUrl']}',";

                    $inverseOperatorsStr = "'" . implode("', '", $inverseOperators) . "'";
                    if (empty($inverseOperators)) $inverseOperatorsStr = '';

                    $freeDeliveryOperatorsStr = "'" . implode("', '", $freeDeliveryOperators) . "'";
                    if (empty($freeDeliveryOperators)) $freeDeliveryOperatorsStr = '';

                    $freeDlvMinCourier = '';
                    if (floatval($settings['freeDlvMinCourier']) > 0) $freeDlvMinCourier = 'free_delivery_min_courier: ' . floatval($settings['freeDlvMinCourier']) . ',';
                    $freeDlvMinPVZ = '';
                    if (floatval($settings['freeDlvMinPVZ']) > 0) $freeDlvMinPVZ = 'free_delivery_min_pvz: ' . floatval($settings['freeDlvMinPVZ']) . ',';

                    // operators_settings
                    $operatorSettingsStr = '';
                    foreach ($operatorsSettings as $st) {
                        if (!empty($operatorSettingsStr)) $operatorSettingsStr .= ',';
                        $operatorSettingsStr .= '{"operator":"'.$st['operator'].'", "inverse_delivery_type": '.($st['inverse_delivery_type']?'true':'false').', "free_delivery": '.($st['free_delivery']?'true':'false').'}';
                    }

                    $isCard = false;
                    $isCash = false;
                    $serviceFilters = '';
                    if (is_array($settings['payCard']) && in_array($selectedPaymentMethod, $settings['payCard'])) {
                        $isCard = true;
                        $serviceFilters = 'NP,COD';
                    }
                    if (is_array($settings['payNal']) && in_array($selectedPaymentMethod, $settings['payNal'])) {
                        $isCash = true;
                        $serviceFilters = 'NP,COD';
                    }

                    $geoEmptyMessage = '';
                    if (!empty($settings['geoEmptyMessage'])) {
                        $geoEmptyMessage = "geo_data_empty_message: '{$settings['geoEmptyMessage']}',";
                    }

                    $script = "<script>window.Ctpt_widgetParams = {
                        popup_mode: true,
                        service_path: '/?rest_route=/catapultodelivery/widgethandler',
                        sender_contact_params: {
                            locality_id: '{$settings['senderLocalityId']}',
                            zip: '{$settings['senderZip']}',
                            cityFrom: '{$settings['senderCity']}',
                        },
                        dadata_token: '{$settings['dadataApikey']}',
                        {$widgetDeliveryTypes}
                        startTabMap: ".(($settings['enMapOpenMode'] === 'map')?'true':'false').",
                        delivery_type: '{$settings['widgetDeliveryFrom']}',
                        day_shift: ".intval($settings['termIncrease']).",
                        inverse_delivery_type_for_operators: [{$inverseOperatorsStr}],
                        free_delivery_operators: [{$freeDeliveryOperatorsStr}],
                        operators_settings: [{$operatorSettingsStr}],
                        location: {
                            address: '',
                            //city: '',
                            //settlement:'',
                        },
                        cargo: {
                            weight: {$cargo['WEI']},
                            height: {$cargo['H']},
                            length: {$cargo['L']},
                            width: {$cargo['W']},
                            quantity: 1,
                        },
                        need_insurance: ".(($settings['mindEnsurance'] === 'yes')?'true':'false').",
                        insured_value: {$cargo['TOTAL']},
                        {$customWSS}
                        
                        services_filter: '{$serviceFilters}', // NP,COD
                        filter_cash: ".($isCash?'true':'false').",
                        filter_card: ".($isCard?'true':'false').",
                        
                        {$geoEmptyMessage}
                        {$freeDlvMinCourier}
                        {$freeDlvMinPVZ}
    
                    };
                    window.Ctpt_runOnStart = ".(($settings['runWidgetOnStart'] === 'yes')?'true':'false').";               

                    if (typeof(window.ctpt_widgetRunned) === 'undefined') window.ctpt_widgetRunned = false;
                    if (window.Ctpt_runOnStart && !window.ctpt_widgetRunned) {
                        window.ctpt_widgetRunned = true;
                        setTimeout(()=>{
                             window.Ctpt_RunWidget();
                        }, 2000);
                    }
                    </script>";

                    $label = $label . '<span class="ctpt_wrapper"><button type="button" class="button alt" id="ctpt_runwidget">Выбрать вариант доставки</button></span>'.$script;
                }
            }
        }

        return $label;
    }

    public function actionWoocommerceAfterCheckoutValidation($fields, $errors) {
        if(in_array(self::PLUGIN_ID . ':pickup', $fields['shipping_method'])) {
            $shippingMethod = new CatapultoShippingMethod();
            $isEnableNoRate = $shippingMethod->isEnabledEmptyRateOrder();

            $isPSChanged = false;
            $ctptRateId = 0;
            if (isset($_POST['ctpt_changed_ps'])) $isPSChanged = sanitize_text_field($_POST['ctpt_changed_ps']) === 'true';
            if (isset($_POST['ctpt_selected_rate'])) $ctptRateId = intval(sanitize_text_field($_POST['ctpt_selected_rate']));

            if (!$isEnableNoRate) {
                if ($ctptRateId == 0) {
                    $errors->add('validation', 'Необходимо выбрать вариант доставки Catapulto');
                    return;
                }
                $cartVariantsDB = new CartVariantsDB();
                $variant = $cartVariantsDB->getDataByCtptrate($ctptRateId);
                if (!$variant) {
                    $errors->add('validation', 'Необходимо выбрать вариант доставки Catapulto');
                    return;
                }
                $variant = unserialize($variant['variant']);
                if (!$variant || !isset($variant['variant'])) {
                    $errors->add('validation', 'Необходимо выбрать вариант доставки Catapulto');
                    return;
                }

                if ($shippingMethod->isNeedReselectAfterChangePS() && $isPSChanged) {
                    $errors->add('validation', 'После изменения способа оплаты Необходимо выбрать новый вариант доставки Catapulto');
                }
            }
        }
    }

    /**
     * Saving additional meta about rate after creating order
     */
    public function actionWoocommerceCheckoutCreateOrder($order, $data) {
        $emptyRateData = [
            'dadata' => [
                'data' => [],
            ],
            'variant' => [
                'additional_services' => [],
                'operator' => 'NotSet',
            ],
            'ctpt_params' => [
                'isEmpty' => true,
            ],
        ];
        if(!empty($_POST['ctpt_selected_rate'])) {
            $ctptRateId = intval(sanitize_text_field($_POST['ctpt_selected_rate']));
            $cartVariantsDB = new CartVariantsDB();
            $variant = $cartVariantsDB->getDataByCtptrate($ctptRateId);
            $variant = unserialize($variant['variant']);
            $order->update_meta_data('ctpt_ratedata', json_encode($variant));
        } else $order->update_meta_data('ctpt_ratedata', json_encode($emptyRateData));
    }

}
