<?php

namespace Ipol\Catapulto\Wordpress\Controller;

use Ipol\Catapulto\Api\Entity\Response\ShipmentId;
use Ipol\Catapulto\Core\Order\Order;
use Ipol\Catapulto\Wordpress\DB\OrdersDB;
use Ipol\Catapulto\Wordpress\Tools\TemplateRender;

class MainController extends AbstractController
{

    const STATUS_COMPLETE = 'completed';
    const STATUS_REJECTED = 'rejected';
    const WP_PAID_STATUS = 'wc-completed';

    public function getWSToken()
    {
        $response = $this->app->getWSToken();
        if ($this->isResponseError($response)) return false;
        return $response->getResponse();
    }

    public function getGeo(
        string $term = '',
        string $cityName = '',
        string $iso = 'ru',
        int $limit = 1,
        ?string $cityFiasId = null,
        ?string $settlementFiasId = null,
        ?string $fiasLevel = null,
        ?string $settlementType = null
    )
    {
        $response = $this->app->geo($term, $cityName, $iso, $limit, $cityFiasId, $settlementFiasId, $fiasLevel, $settlementType);
        if ($this->isResponseError($response)) return false;
        return $response->getResponse()->getGeo();
    }

    public function createCargo(Order $order)
    {
        $response = $this->app->cargoCreate($order);
        if ($this->isResponseError($response)) return false;
        return $response->getResponse();
    }

    public function getCompanyIcons()
    {
        $response = $this->app->companyIcon();
        if ($this->isResponseError($response)) return false;
        return $response->getResponse();
    }

    public function createRate(Order $order)
    {
        $response = $this->app->rateCreate($order);
        if ($this->isResponseError($response)) return false;
        return $response->getResponse();
    }

    public function getRate(
        string $rateId,
        array $filter,
        bool $needInsuranse = false,
        float $insuranceValue = 0
    )
    {
        $response = $this->app->rateRead(
            $rateId,
            $filter['pickup_days_shift'] ?? 0,
            [$filter['shipping_type_filter']],
            ($filter['services_filter']) ? [$filter['services_filter']] : [],
            $needInsuranse,
            $insuranceValue
        );
        if ($this->isResponseError($response)) return false;
        return $response->getResponse();
    }

    public function getTarif($tarifId, int $daysShift = 0)
    {
        $response = $this->app->tariffRead($tarifId, $daysShift);
        if ($this->isResponseError($response)) return false;
        return $response->getResponse();
    }

    public function getTerminals(Order $order)
    {
        $response = $this->app->terminalList($order);
        if ($this->isResponseError($response)) return false;
        return $response->getResponse();
    }

    public function getTerminal(int $terminalId)
    {
        $response = $this->app->terminalRead($terminalId);
        if ($this->isResponseError($response)) return false;
        return $response->getResponse();
    }

    public function getOrderInfo(string $catapultoOrderKey)
    {
        $response = $this->app->shipmentRead($catapultoOrderKey);
        if ($this->isResponseError($response)) return false;
        /** @var ShipmentId $orderData */
        $catapultoOrder = $response->getResponse();

        $pod = $catapultoOrder->getPod();
        if ($pod) $pod = (string) $pod['total_value'];

        $ordersDB = new OrdersDB();
        $currentData = $ordersDB->getByColumn('number', $catapultoOrder->getNumber());

        $orderData = [
            'catapulto_id' => $catapultoOrder->getId(),
            'number' => $catapultoOrder->getNumber(),
            'tracking_number' => $catapultoOrder->getTrackingNumber() ?? '',
            'tracking_link' => $catapultoOrder->getTrackingLink() ?? '',
            'last_tracking_text' => $catapultoOrder->getLastTrackingText() ?? '',
            'main_status' => $catapultoOrder->getMainStatus(),
            'main_status_display' => $catapultoOrder->getMainStatusDisplay(),
            'pickup_day' => $catapultoOrder->getPickupDay(),
            'delivery_day' => $catapultoOrder->getDeliveryDay(),
            'price' => $catapultoOrder->getPrice(),
            'weight' => $catapultoOrder->getWeight(),
            'operator' => $catapultoOrder->getOperator(),
            'problem_text' => $catapultoOrder->getProblemText() ?? '',
            'tracking_status' => $catapultoOrder->getTrackingStatus() ?? '',
            'pod' => $pod ?? '',
            'documents' => serialize($catapultoOrder->getDocuments()->getFields()),
        ];
        $ordersDB->saveOrUpdate($orderData, 'number');

        //Send notification email to client
        if (
            !empty($orderData['tracking_link'])
            && empty($currentData['tracking_link'])
            && ($this->emailNotifyEnabled === true)
            && (!empty($currentData['receiver_email']))
        ) {
            $render = new TemplateRender();
            wp_mail(
                $currentData['receiver_email'],
                'Вашему отправлению присвоен трек-номер',
                $render->render('mail/newTrackNum.tpl', ['tracklink' => $orderData['tracking_link']]),
                ['Content-type: text/html; charset=utf-8']
            );
        }

        //Working with statuses....
        if (
            ($orderData['tracking_status'] != $currentData['tracking_status'])
            && isset($this->orderMapping[$orderData['tracking_status']])
            && ($this->orderMapping[$orderData['tracking_status']] != 'not_set')
        ) {
            $wpOrder = new \WC_Order($currentData['order_id']);
            $wpOrderStatus = $wpOrder->get_status();
            if ($wpOrderStatus != $this->orderMapping[$orderData['tracking_status']]) {
                $wpOrder->update_status($this->orderMapping[$orderData['tracking_status']]);
                $wpOrder->save();
            }
        }

        //Make delivered order - "paid" status
        if ( $this->markDeliveriedOrderIsPaid && ($orderData['tracking_status'] === self::STATUS_COMPLETE) ) {
            $wpOrder = new \WC_Order($currentData['order_id']);
            $wpOrderStatus = $wpOrder->get_status();
            if ($wpOrderStatus != self::WP_PAID_STATUS ) {
                $wpOrder->update_status(self::WP_PAID_STATUS);
                $wpOrder->save();
            }
        }

        return [
            'currentData' => $catapultoOrder,
            'oldData' => $currentData,
        ];
    }

    public function cancelOrder(string $catapultoOrderKey)
    {
        $response = $this->app->cancelOrder($catapultoOrderKey);
        if ($this->isResponseError($response)) return false;
        return $response->getResponse()->getStatus();
    }


}
